import com.github.tomakehurst.wiremock.WireMockServer;
import com.github.tomakehurst.wiremock.junit.WireMockRule;
import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.JsonNode;
import com.mashape.unirest.http.exceptions.UnirestException;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import java.io.IOException;
import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static com.github.tomakehurst.wiremock.core.WireMockConfiguration.wireMockConfig;
import static org.junit.Assert.*;

public class YouAreEllTest {


    private WireMockServer wireMockServer ; // No-args constructor defaults
    private YouAreEll el;

    @Before
    public void setUp() throws IOException {
        wireMockServer = new WireMockServer(wireMockConfig().dynamicPort());
        wireMockServer.start();
        el = new YouAreEll("http://localhost:" + wireMockServer.port());
    }

    @After
    public void tearDown(){
        wireMockServer.stop();
    }

    @Test
    public void makeCallGetIds() throws UnirestException, IOException {
       Ids id = new Ids("-","test","loopdeeloop");
        String expected = "{\"github\":\"test\",\"name\":\"loopdeeloop\",\"userid\":\"-\"}";
        wireMockServer.stubFor(get(urlEqualTo("/ids"))
                .willReturn(aResponse()
                                .withStatus(200)
                               .withBody(el.convertToJson(id))));
       HttpResponse<JsonNode> result = el.MakeURLCall("/ids","GET", "");
       // verify(getRequestedFor(urlEqualTo("/ids")));
        assertEquals(200, result.getStatus());
        assertEquals(expected, result.getBody().toString());
    }

    @Test
    public void makeCallGetId() throws UnirestException {
        wireMockServer.stubFor(get(urlEqualTo("/ids"))
                .willReturn(aResponse()
                        .withStatus(200)));
        HttpResponse<JsonNode> jsonResponse = el.MakeURLCall("/ids","GET", "");
        assertEquals(200, jsonResponse.getStatus());
    }

    @Test
    public void makeCallGetMessages() throws UnirestException {
        wireMockServer.stubFor(get(urlEqualTo("/messages"))
                .willReturn(aResponse()
                        .withStatus(200)));
        HttpResponse<JsonNode> jsonResponse = el.MakeURLCall("/messages","GET", "");
        assertEquals(200, jsonResponse.getStatus());
    }

    @Test
    public void makeCallpostIds() throws UnirestException {
        Ids id = new Ids("-","test","loopdeeloop");
        wireMockServer.stubFor(get(urlEqualTo("/ids"))
                .willReturn(aResponse()
                        .withStatus(200)));
        HttpResponse<JsonNode> jsonResponse = el.MakeURLCall("/ids","POST", el.convertToJson(id) );
        assertEquals(200, jsonResponse.getStatus());
    }

    @Test
    public void makeCallputIds() throws UnirestException {
        String expected = "[\"userid\":\"fc1a092b66da0f698575483d88985e9864cbadda\",\"name\":\"loopdeeloop\",\"github\":\"bester\"]";
        Ids id = new Ids("fc1a092b66da0f698575483d88985e9864cbadda","bester","loopdeeloop");
        System.out.println(el.convertToJson(id));
        //String expected = "";
        wireMockServer.stubFor(get(urlEqualTo("/ids"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withBody(el.convertToJson(id))));
        HttpResponse<JsonNode> jsonResponse = el.MakeURLCall("/ids","PUT", expected );
        assertEquals(200, jsonResponse.getStatus());
      //  assertEquals(expected, jsonResponse.getBody().toString());
    }

    @Test
    public void get_ids() throws UnirestException {
        wireMockServer.stubFor(get(urlEqualTo("/ids"))
                .willReturn(aResponse()
                        .withStatus(200)));
        assertNotNull(el.get_ids());
    }

    @Test
    public void create_idsTest() throws UnirestException {
        wireMockServer.stubFor(get(urlEqualTo("/ids"))
                .willReturn(aResponse()
                        .withStatus(200)));
        Ids id = new Ids("-","test","loopdeeloop");
        assertNotNull(el.create_id(id));
    }

    @Test
    public void replace_idsTest() throws UnirestException {
            Ids id = new Ids("-","test","loopdeeloopwoop");
            assertNotNull(el.replace_id(id));
    }

    @Test
    public void get_messages() throws UnirestException {
        assertNotNull(el.get_messages());
    }

    @Test
    public void send_messages() throws UnirestException {
        Ids id = new Ids("-","test","testyTest");
        assertNotNull(el.create_id(id));
    }
}